use std::collections::BTreeMap;
use std::sync::Arc;
use std::sync::Mutex;

use sqlx::Pool;
use sqlx::Sqlite;
use sqlx::SqlitePool;
use sqlx::{self, sqlite};

#[derive(Clone)]
pub struct CaptchaState {
    pub capthca_db: Arc<Mutex<BTreeMap<i32, String>>>,
}

impl CaptchaState {
    pub fn new() -> Self {
        Self {
            capthca_db: Arc::new(Mutex::new(BTreeMap::new())),
        }
    }
}

#[derive(Clone)]
pub struct ServerState {
    pub pool: Pool<Sqlite>,
}

impl ServerState {
    pub async fn new() -> Self {
        // This is almost certainly bad practice for more reasons than I can count
        dotenvy::dotenv().ok();
        let db_url = dotenvy::var("DATABASE_URL").unwrap_or(":memory:".to_string());

        let pool = sqlite::SqlitePoolOptions::new()
            .max_connections(5)
            .connect(&db_url)
            .await
            .unwrap();

        sqlx::migrate!("./migrations").run(&pool).await.unwrap();

        #[cfg(debug_assertions)]
        debug_setup(&pool).await.unwrap();

        Self { pool }
    }
}

// Inserts a bunch of dummy data into the database
// Mostly useful for debugging new posts, as we need to satisfy foreign key constraints.
#[cfg(debug_assertions)]
async fn debug_setup(pool: &SqlitePool) -> Result<(), sqlx::Error> {
    use sqlx::query;

    use crate::db::db_new_user;

    db_new_user("user".to_string(), "pass".to_string(), pool).await;

    // Check if the demo post already exists
    let posted = query!("SELECT * FROM posts WHERE id = 1",)
        .fetch_one(pool)
        .await
        .ok();

    // If the demo user already has a post, don't insert another one
    if !posted.is_some() {
        // This requires that the user with id 1 exists in the user table
        query!("INSERT INTO posts (user_id, content) VALUES (1, 'Hello world! The demo username is user and the password is pass.')",)
        .execute(pool)
        .await?;
    }

    Ok(())
}
